﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.ConditionActionEncoding {
  [Item("CoveringSolutionCreator", "Description missing")]
  [StorableClass]
  public abstract class CoveringSolutionCreator : SingleSuccessorOperator, ICoveringSolutionCreator {

    #region Parameter Properties
    public ILookupParameter<IInput> CoverInputParameter {
      get { return (ILookupParameter<IInput>)Parameters["CoverInput"]; }
    }
    public ILookupParameter<IAction> ActionParameter {
      get { return (ILookupParameter<IAction>)Parameters["Action"]; }
    }
    public IValueLookupParameter<IClassifier> CreatedClassifierParameter {
      get { return (IValueLookupParameter<IClassifier>)Parameters["CreatedClassifier"]; }
    }
    public ILookupParameter<PercentValue> ChangeSymbolProbabilityParameter {
      get { return (ILookupParameter<PercentValue>)Parameters["ChangeSymbolProbability"]; }
    }
    public ILookupParameter<IRandom> RandomParameter {
      get { return (ILookupParameter<IRandom>)Parameters["Random"]; }
    }
    #endregion

    [StorableConstructor]
    protected CoveringSolutionCreator(bool deserializing) : base(deserializing) { }
    protected CoveringSolutionCreator(CoveringSolutionCreator original, Cloner cloner)
      : base(original, cloner) {
    }
    public CoveringSolutionCreator()
      : base() {
      Parameters.Add(new LookupParameter<IInput>("CoverInput"));
      Parameters.Add(new LookupParameter<IAction>("Action"));
      Parameters.Add(new ValueLookupParameter<IClassifier>("CreatedClassifier"));
      Parameters.Add(new LookupParameter<PercentValue>("ChangeSymbolProbability"));
      Parameters.Add(new LookupParameter<IRandom>("Random"));
    }

    public override IOperation Apply() {
      CreatedClassifierParameter.ActualValue = CreateCoveredClassifier(CoverInputParameter.ActualValue, ActionParameter.ActualValue, RandomParameter.ActualValue, ChangeSymbolProbabilityParameter.ActualValue.Value);
      return base.Apply();
    }

    protected abstract IClassifier CreateCoveredClassifier(IInput input, IAction action, IRandom random, double changeSymbolProbability);
  }
}
